// Copyright (c) 2019 IoTeX
// This is an alpha (internal) release and is not suitable for production. This source code is provided 'as is' and no
// warranties are given as to title or non-infringement, merchantability or fitness for purpose and, to the extent
// permitted by law, all liability for your use of the code is disset epoch rewarded. This source code is governed by Apache
// License 2.0 that can be found in the LICENSE file.

package action

import (
	"math/big"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestDonateToRewardingFund(t *testing.T) {
	b := DepositToRewardingFundBuilder{}
	s1 := b.SetAmount(big.NewInt(1)).
		SetData([]byte{2}).
		Build()
	proto := s1.Proto()
	s2 := DepositToRewardingFund{}
	require.NoError(t, s2.LoadProto(proto))
	assert.Equal(t, s1.Amount(), s2.Amount())
	assert.Equal(t, s2.Data(), s2.Data())
}

func TestClaimFromRewardingFund(t *testing.T) {
	b := ClaimFromRewardingFundBuilder{}
	s1 := b.SetAmount(big.NewInt(1)).
		SetData([]byte{2}).
		Build()
	proto := s1.Proto()
	s2 := ClaimFromRewardingFund{}
	require.NoError(t, s2.LoadProto(proto))
	assert.Equal(t, s1.Amount(), s2.Amount())
	assert.Equal(t, s2.Data(), s2.Data())
}

func TestGrantBlockReward(t *testing.T) {
	b := GrantRewardBuilder{}
	s1 := b.SetRewardType(BlockReward).SetHeight(1).Build()
	proto := s1.Proto()
	s2 := GrantReward{}
	require.NoError(t, s2.LoadProto(proto))
	assert.Equal(t, s1.RewardType(), s2.RewardType())
}
